@@ -295,8 +295,12 @@ sms_done_cb (void *cls,
 {
   struct ANASTASIS_AUTHORIZATION_State *as = cls;
 
-  as->child = NULL;
   as->cwh = NULL;
+  if (NULL != as->child)
+  {
+    GNUNET_OS_process_destroy (as->child);
+    as->child = NULL;
+  }
   as->pst = type;
   as->exit_code = exit_code;
   MHD_resume_connection (as->connection);
