@@ -174,11 +174,6 @@ extern struct TALER_Amount AH_question_cost;
  */
 extern char *AH_backend_url;
 
-/**
- * Taler currency.
- */
-extern char *AH_currency;
-
 /**
  * Heap for processing timeouts of requests.
  */
@@ -205,9 +200,9 @@ extern char *AH_fulfillment_url;
 extern char *AH_business_name;
 
 /**
- * Our server salt.
+ * Our provider salt.
  */
-extern struct ANASTASIS_CRYPTO_ProviderSaltP AH_server_salt;
+extern struct ANASTASIS_CRYPTO_ProviderSaltP AH_provider_salt;
 
 /**
  * Our context for making HTTP requests.
