@@ -425,8 +425,8 @@ check_payment_cb (void *cls,
   GNUNET_assert (MHD_HTTP_OK == hr->http_status);
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Payment status checked: %d\n",
-              osr->details.success.status);
-  switch (osr->details.success.status)
+              osr->details.ok.status);
+  switch (osr->details.ok.status)
   {
   case TALER_MERCHANT_OSC_PAID:
     {
@@ -441,7 +441,7 @@ check_payment_cb (void *cls,
         GNUNET_JSON_spec_end ()
       };
 
-      contract = osr->details.success.details.paid.contract_terms;
+      contract = osr->details.ok.details.paid.contract_terms;
       if (GNUNET_OK !=
           GNUNET_JSON_parse (contract,
                              cspec,
@@ -558,6 +558,7 @@ await_payment (struct PolicyUploadContext *puc)
 static MHD_RESULT
 begin_payment (struct PolicyUploadContext *puc)
 {
+  static const char *no_uuids[1] = { NULL };
   json_t *order;
 
   GNUNET_CONTAINER_DLL_insert (puc_head,
@@ -608,7 +609,7 @@ begin_payment (struct PolicyUploadContext *puc)
                                          0,
                                          NULL, /* no inventory products */
                                          0,
-                                         NULL, /* no uuids */
+                                         no_uuids, /* no uuids */
                                          false, /* do NOT require claim token */
                                          &proposal_cb,
                                          puc);
@@ -662,34 +663,10 @@ AH_handler_policy_post (
     hc->cc = &cleanup_ctx;
     puc->con = connection;
 
-    {
-      const char *pay_id;
-
-      pay_id = MHD_lookup_connection_value (connection,
-                                            MHD_HEADER_KIND,
-                                            ANASTASIS_HTTP_HEADER_PAYMENT_IDENTIFIER);
-      if (NULL != pay_id)
-      {
-        if (GNUNET_OK !=
-            GNUNET_STRINGS_string_to_data (
-              pay_id,
-              strlen (pay_id),
-              &puc->payment_identifier,
-              sizeof (struct ANASTASIS_PaymentSecretP)))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             ANASTASIS_HTTP_HEADER_PAYMENT_IDENTIFIER
-                                             " header must be a base32-encoded Payment-Secret");
-        }
-        puc->payment_identifier_provided = true;
-        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                    "Policy upload started with payment identifier `%s'\n",
-                    pay_id);
-      }
-    }
+    TALER_MHD_parse_request_header_auto (connection,
+                                         ANASTASIS_HTTP_HEADER_PAYMENT_IDENTIFIER,
+                                         &puc->payment_identifier,
+                                         puc->payment_identifier_provided);
     puc->account = *account_pub;
 
     /* check for meta-data */
@@ -768,28 +745,10 @@ AH_handler_policy_post (
       }
       puc->upload_size = (size_t) len;
     }
-    {
-      /* Check if header contains Anastasis-Policy-Signature */
-      const char *sig_s;
 
-      sig_s = MHD_lookup_connection_value (connection,
-                                           MHD_HEADER_KIND,
-                                           ANASTASIS_HTTP_HEADER_POLICY_SIGNATURE);
-      if ( (NULL == sig_s) ||
-           (GNUNET_OK !=
-            GNUNET_STRINGS_string_to_data (sig_s,
-                                           strlen (sig_s),
-                                           &puc->account_sig,
-                                           sizeof (puc->account_sig))) )
-      {
-        GNUNET_break_op (0);
-        return TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_BAD_REQUEST,
-                                           TALER_EC_ANASTASIS_POLICY_BAD_SIGNATURE,
-                                           ANASTASIS_HTTP_HEADER_POLICY_SIGNATURE
-                                           " header must include a base32-encoded EdDSA signature");
-      }
-    }
+    TALER_MHD_parse_request_header_auto_t (connection,
+                                           ANASTASIS_HTTP_HEADER_POLICY_SIGNATURE,
+                                           &puc->account_sig);
     {
       /* Check if header contains an ETAG */
       const char *etag;
@@ -798,9 +757,12 @@ AH_handler_policy_post (
                                           MHD_HEADER_KIND,
                                           MHD_HTTP_HEADER_IF_NONE_MATCH);
       if ( (NULL == etag) ||
+           (2 >= strlen (etag)) ||
+           ('"' != etag[0]) ||
+           ('"' != etag[strlen (etag) - 1]) ||
            (GNUNET_OK !=
-            GNUNET_STRINGS_string_to_data (etag,
-                                           strlen (etag),
+            GNUNET_STRINGS_string_to_data (etag + 1,
+                                           strlen (etag) - 2,
                                            &puc->new_policy_upload_hash,
                                            sizeof (puc->new_policy_upload_hash))) )
       {
@@ -834,39 +796,10 @@ AH_handler_policy_post (
       }
     }
 
-    {
-      const char *long_poll_timeout_ms;
-
-      long_poll_timeout_ms = MHD_lookup_connection_value (connection,
-                                                          MHD_GET_ARGUMENT_KIND,
-                                                          "timeout_ms");
-      if (NULL != long_poll_timeout_ms)
-      {
-        unsigned int timeout;
-        char dummy;
-
-        if (1 != sscanf (long_poll_timeout_ms,
-                         "%u%c",
-                         &timeout,
-                         &dummy))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             "timeout_ms (must be non-negative number)");
-        }
-        puc->timeout
-          = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_relative_multiply (
-                                                GNUNET_TIME_UNIT_MILLISECONDS,
-                                                timeout));
-      }
-      else
-      {
-        puc->timeout = GNUNET_TIME_relative_to_absolute (
-          CHECK_PAYMENT_GENERIC_TIMEOUT);
-      }
-    }
+    puc->timeout = GNUNET_TIME_relative_to_absolute (
+      CHECK_PAYMENT_GENERIC_TIMEOUT);
+    TALER_MHD_parse_request_timeout (connection,
+                                     &puc->timeout);
 
     /* check if the client insists on paying */
     {
