@@ -360,22 +360,17 @@ AH_truth_challenge_shutdown (void)
  * Callback to process a POST /orders/ID/refund request
  *
  * @param cls closure with a `struct RefundEntry *`
- * @param hr HTTP response details
- * @param taler_refund_uri the refund uri offered to the wallet
- * @param h_contract hash of the contract a Browser may need to authorize
- *        obtaining the HTTP response.
+ * @param rr response details
  */
 static void
 refund_cb (
   void *cls,
-  const struct TALER_MERCHANT_HttpResponse *hr,
-  const char *taler_refund_uri,
-  const struct TALER_PrivateContractHashP *h_contract)
+  const struct TALER_MERCHANT_RefundResponse *rr)
 {
   struct RefundEntry *re = cls;
 
   re->ro = NULL;
-  switch (hr->http_status)
+  switch (rr->hr.http_status)
   {
   case MHD_HTTP_OK:
     {
@@ -407,9 +402,9 @@ refund_cb (
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Refund `%s' failed with HTTP status %u: %s (#%u)\n",
                 re->order_id,
-                hr->http_status,
-                hr->hint,
-                (unsigned int) hr->ec);
+                rr->hr.http_status,
+                rr->hr.hint,
+                (unsigned int) rr->hr.ec);
     break;
   }
   GNUNET_CONTAINER_DLL_remove (re_head,
@@ -713,7 +708,7 @@ check_payment_cb (void *cls,
   }
 
   GNUNET_assert (MHD_HTTP_OK == hr->http_status);
-  switch (osr->details.success.status)
+  switch (osr->details.ok.status)
   {
   case TALER_MERCHANT_OSC_PAID:
     {
@@ -804,6 +799,7 @@ begin_payment (struct ChallengeContext *gc)
   else
   {
     /* Create a fresh order */
+    static const char *no_uuids[1] = { NULL };
     json_t *order;
     struct GNUNET_TIME_Timestamp pay_deadline;
 
@@ -837,7 +833,7 @@ begin_payment (struct ChallengeContext *gc)
                                           0,
                                           NULL, /* no inventory products */
                                           0,
-                                          NULL, /* no uuids */
+                                          no_uuids, /* no uuids */
                                           false, /* do NOT require claim token */
                                           &proposal_cb,
                                           gc);
