@@ -555,7 +555,7 @@ check_payment_cb (void *cls,
   }
 
   GNUNET_assert (MHD_HTTP_OK == hr->http_status);
-  switch (osr->details.success.status)
+  switch (osr->details.ok.status)
   {
   case TALER_MERCHANT_OSC_PAID:
     {
@@ -646,6 +646,7 @@ begin_payment (struct SolveContext *gc)
   else
   {
     /* Create a fresh order */
+    static const char *no_uuids[1] = { NULL };
     json_t *order;
     struct GNUNET_TIME_Timestamp pay_deadline;
 
@@ -679,7 +680,7 @@ begin_payment (struct SolveContext *gc)
                                           0,
                                           NULL, /* no inventory products */
                                           0,
-                                          NULL, /* no uuids */
+                                          no_uuids, /* no uuids */
                                           false, /* do NOT require claim token */
                                           &proposal_cb,
                                           gc);
@@ -1140,40 +1141,10 @@ AH_handler_truth_solve (
     gc->connection = connection;
     gc->truth_uuid = *truth_uuid;
     gc->hc->cc = &request_done;
-
-    {
-      const char *long_poll_timeout_ms;
-
-      long_poll_timeout_ms = MHD_lookup_connection_value (connection,
-                                                          MHD_GET_ARGUMENT_KIND,
-                                                          "timeout_ms");
-      if (NULL != long_poll_timeout_ms)
-      {
-        unsigned int timeout;
-        char dummy;
-
-        if (1 != sscanf (long_poll_timeout_ms,
-                         "%u%c",
-                         &timeout,
-                         &dummy))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             "timeout_ms (must be non-negative number)");
-        }
-        gc->timeout
-          = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_relative_multiply (
-                                                GNUNET_TIME_UNIT_MILLISECONDS,
-                                                timeout));
-      }
-      else
-      {
-        gc->timeout = GNUNET_TIME_relative_to_absolute (
-          GNUNET_TIME_UNIT_SECONDS);
-      }
-    }
+    gc->timeout = GNUNET_TIME_relative_to_absolute (
+      GNUNET_TIME_UNIT_SECONDS);
+    TALER_MHD_parse_request_timeout (connection,
+                                     &gc->timeout);
   } /* end of first-time initialization (if NULL == gc) */
   else
   {
