@@ -310,7 +310,7 @@ check_payment_cb (void *cls,
       NULL);
     break;
   case MHD_HTTP_OK:
-    switch (osr->details.success.status)
+    switch (osr->details.ok.status)
     {
     case TALER_MERCHANT_OSC_PAID:
       {
@@ -325,7 +325,7 @@ check_payment_cb (void *cls,
           GNUNET_JSON_spec_end ()
         };
 
-        contract = osr->details.success.details.paid.contract_terms;
+        contract = osr->details.ok.details.paid.contract_terms;
         if (GNUNET_OK !=
             GNUNET_JSON_parse (contract,
                                cspec,
@@ -354,7 +354,7 @@ check_payment_cb (void *cls,
         qs = db->record_truth_upload_payment (
           db->cls,
           &tuc->truth_uuid,
-          &osr->details.success.details.paid.deposit_total,
+          &osr->details.ok.details.paid.deposit_total,
           paid_until);
         if (qs <= 0)
         {
@@ -396,6 +396,7 @@ check_payment_cb (void *cls,
   case MHD_HTTP_NOT_FOUND:
     /* Setup fresh order */
     {
+      static const char *no_uuids[1] = { NULL };
       char *order_id;
       json_t *order;
 
@@ -426,7 +427,7 @@ check_payment_cb (void *cls,
                                              0,
                                              NULL, /* no inventory products */
                                              0,
-                                             NULL, /* no uuids */
+                                             no_uuids, /* no uuids */
                                              false, /* do NOT require claim token */
                                              &proposal_cb,
                                              tuc);
@@ -549,78 +550,12 @@ AH_handler_truth_post (
     tuc->truth_uuid = *truth_uuid;
     hc->ctx = tuc;
     hc->cc = &cleanup_truth_post;
-
-    /* check for excessive upload */
-    {
-      const char *lens;
-      unsigned long len;
-      char dummy;
-
-      lens = MHD_lookup_connection_value (connection,
-                                          MHD_HEADER_KIND,
-                                          MHD_HTTP_HEADER_CONTENT_LENGTH);
-      if ( (NULL == lens) ||
-           (1 != sscanf (lens,
-                         "%lu%c",
-                         &len,
-                         &dummy)) )
-      {
-        GNUNET_break_op (0);
-        return TALER_MHD_reply_with_error (
-          connection,
-          MHD_HTTP_BAD_REQUEST,
-          (NULL == lens)
-          ? TALER_EC_ANASTASIS_GENERIC_MISSING_CONTENT_LENGTH
-          : TALER_EC_ANASTASIS_GENERIC_MALFORMED_CONTENT_LENGTH,
-          NULL);
-      }
-      if (len / 1024 / 1024 >= AH_upload_limit_mb)
-      {
-        GNUNET_break_op (0);
-        return TALER_MHD_reply_with_error (connection,
-                                           MHD_HTTP_PAYLOAD_TOO_LARGE,
-                                           TALER_EC_SYNC_MALFORMED_CONTENT_LENGTH,
-                                           "Content-length value not acceptable");
-      }
-    }
-
-    {
-      const char *long_poll_timeout_ms;
-
-      long_poll_timeout_ms = MHD_lookup_connection_value (connection,
-                                                          MHD_GET_ARGUMENT_KIND,
-                                                          "timeout_ms");
-      if (NULL != long_poll_timeout_ms)
-      {
-        unsigned int timeout;
-        char dummy;
-
-        if (1 != sscanf (long_poll_timeout_ms,
-                         "%u%c",
-                         &timeout,
-                         &dummy))
-        {
-          GNUNET_break_op (0);
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_BAD_REQUEST,
-                                             TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                             "timeout_ms (must be non-negative number)");
-        }
-        tuc->timeout
-          = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_relative_multiply (
-                                                GNUNET_TIME_UNIT_MILLISECONDS,
-                                                timeout));
-        GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                    "Long polling for %u ms enabled\n",
-                    timeout);
-      }
-      else
-      {
-        tuc->timeout = GNUNET_TIME_relative_to_absolute (
-          GNUNET_TIME_UNIT_SECONDS);
-      }
-    }
-
+    TALER_MHD_check_content_length (connection,
+                                    AH_upload_limit_mb * 1024LLU * 1024LLU);
+    tuc->timeout = GNUNET_TIME_relative_to_absolute (
+      GNUNET_TIME_UNIT_SECONDS);
+    TALER_MHD_parse_request_timeout (connection,
+                                     &tuc->timeout);
   } /* end 'if (NULL == tuc)' */
 
   if (NULL != tuc->resp)
