@@ -21,7 +21,7 @@
  * @author Dennis Neufeld
  */
 #include <jansson.h>
-#include <gnunet/gnunet_crypto_lib.h>
+#include <gnunet/gnunet_util_lib.h>
 
 
 /**
