@@ -612,23 +612,22 @@ ANASTASIS_challenge_abort (struct ANASTASIS_Challenge *c)
  * Function called with the results of a #ANASTASIS_policy_lookup()
  *
  * @param cls closure
- * @param http_status HTTp status code.
  * @param dd details about the lookup operation
  */
 static void
 policy_lookup_cb (void *cls,
-                  unsigned int http_status,
                   const struct ANASTASIS_DownloadDetails *dd)
 {
   struct ANASTASIS_Recovery *r = cls;
   void *plaintext;
   size_t size_plaintext;
   json_error_t json_error;
-  json_t *dec_policies;
-  json_t *esc_methods;
+  const json_t *dec_policies;
+  const json_t *esc_methods;
+  json_t *recovery_document;
 
   r->plo = NULL;
-  switch (http_status)
+  switch (dd->http_status)
   {
   case MHD_HTTP_OK:
     break;
@@ -660,7 +659,7 @@ policy_lookup_cb (void *cls,
   default:
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u in %s:%u\n",
-                http_status,
+                dd->http_status,
                 __FILE__,
                 __LINE__);
     r->csc (r->csc_cls,
@@ -670,7 +669,7 @@ policy_lookup_cb (void *cls,
     ANASTASIS_recovery_abort (r);
     return;
   }
-  if (NULL == dd->policy)
+  if (NULL == dd->details.ok.policy)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "No recovery data available");
@@ -682,8 +681,8 @@ policy_lookup_cb (void *cls,
     return;
   }
   ANASTASIS_CRYPTO_recovery_document_decrypt (&r->id,
-                                              dd->policy,
-                                              dd->policy_size,
+                                              dd->details.ok.policy,
+                                              dd->details.ok.policy_size,
                                               &plaintext,
                                               &size_plaintext);
   if (size_plaintext < sizeof (uint32_t))
@@ -698,7 +697,6 @@ policy_lookup_cb (void *cls,
     return;
   }
   {
-    json_t *recovery_document;
     uint32_t be_size;
     uLongf pt_size;
     char *pt;
@@ -761,10 +759,10 @@ policy_lookup_cb (void *cls,
     {
       const char *secret_name = NULL;
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_json ("policies",
-                               &dec_policies),
-        GNUNET_JSON_spec_json ("escrow_methods",
-                               &esc_methods),
+        GNUNET_JSON_spec_array_const ("policies",
+                                      &dec_policies),
+        GNUNET_JSON_spec_array_const ("escrow_methods",
+                                      &esc_methods),
         GNUNET_JSON_spec_mark_optional (
           GNUNET_JSON_spec_string ("secret_name",
                                    &secret_name),
@@ -799,10 +797,9 @@ policy_lookup_cb (void *cls,
         r->ri.secret_name = r->secret_name;
       }
     }
-    json_decref (recovery_document);
   }
 
-  r->ri.version = dd->version;
+  r->ri.version = dd->details.ok.version;
   r->ri.cs_len = json_array_size (esc_methods);
   r->ri.dps_len = json_array_size (dec_policies);
   r->ri.dps = GNUNET_new_array (r->ri.dps_len,
@@ -848,13 +845,12 @@ policy_lookup_cb (void *cls,
                            NULL, NULL))
     {
       GNUNET_break_op (0);
-      json_decref (esc_methods);
-      json_decref (dec_policies);
       r->csc (r->csc_cls,
               ANASTASIS_RS_POLICY_MALFORMED_JSON,
               NULL,
               0);
       ANASTASIS_recovery_abort (r);
+      json_decref (recovery_document);
       return;
     }
     cs->url = GNUNET_strdup (url);
@@ -864,13 +860,11 @@ policy_lookup_cb (void *cls,
     cs->instructions = GNUNET_strdup (instructions);
     cs->ci.instructions = cs->instructions;
   }
-  json_decref (esc_methods);
 
   for (unsigned int j = 0; j < r->ri.dps_len; j++)
   {
     struct DecryptionPolicy *dp = &r->dps[j];
-
-    json_t *uuids = NULL;
+    const json_t *uuids;
     json_t *uuid;
     size_t n_index;
     struct GNUNET_JSON_Specification spec[] = {
@@ -879,27 +873,25 @@ policy_lookup_cb (void *cls,
                                 &dp->emk_size),
       GNUNET_JSON_spec_fixed_auto ("master_salt",
                                    &dp->master_salt),
-      GNUNET_JSON_spec_json ("uuids",
-                             &uuids),
+      GNUNET_JSON_spec_array_const ("uuids",
+                                    &uuids),
       GNUNET_JSON_spec_end ()
     };
 
     r->ri.dps[j] = &r->dps[j].pub_details;
-    if ( (GNUNET_OK !=
-          GNUNET_JSON_parse (json_array_get (dec_policies,
-                                             j),
-                             spec,
-                             NULL, NULL)) ||
-         (! json_is_array (uuids)) )
+    if (GNUNET_OK !=
+        GNUNET_JSON_parse (json_array_get (dec_policies,
+                                           j),
+                           spec,
+                           NULL, NULL))
     {
       GNUNET_break_op (0);
-      json_decref (uuids);
-      json_decref (dec_policies);
       r->csc (r->csc_cls,
               ANASTASIS_RS_POLICY_MALFORMED_JSON,
               NULL,
               0);
       ANASTASIS_recovery_abort (r);
+      json_decref (recovery_document);
       return;
     }
 
@@ -925,13 +917,12 @@ policy_lookup_cb (void *cls,
               sizeof (uuid))) )
       {
         GNUNET_break_op (0);
-        json_decref (dec_policies);
-        json_decref (uuids);
         r->csc (r->csc_cls,
                 ANASTASIS_RS_POLICY_MALFORMED_JSON,
                 NULL,
                 0);
         ANASTASIS_recovery_abort (r);
+        json_decref (recovery_document);
         return;
       }
       for (unsigned int i = 0; i<r->ri.cs_len; i++)
@@ -947,21 +938,19 @@ policy_lookup_cb (void *cls,
       if (! found)
       {
         GNUNET_break_op (0);
-        json_decref (dec_policies);
-        json_decref (uuids);
         r->csc (r->csc_cls,
                 ANASTASIS_RS_POLICY_MALFORMED_JSON,
                 NULL,
                 0);
         ANASTASIS_recovery_abort (r);
+        json_decref (recovery_document);
         return;
       }
     }
-    json_decref (uuids);
   }
-  json_decref (dec_policies);
   r->pc (r->pc_cls,
          &r->ri);
+  json_decref (recovery_document);
 }
 
 
@@ -1134,7 +1123,7 @@ ANASTASIS_recovery_serialize (const struct ANASTASIS_Recovery *r)
  */
 static enum GNUNET_GenericReturnValue
 parse_cs_array (struct ANASTASIS_Recovery *r,
-                json_t *cs_arr)
+                const json_t *cs_arr)
 {
   json_t *cs;
   unsigned int n_index;
@@ -1220,7 +1209,7 @@ parse_cs_array (struct ANASTASIS_Recovery *r,
  */
 static enum GNUNET_GenericReturnValue
 parse_dps_array (struct ANASTASIS_Recovery *r,
-                 json_t *dps_arr)
+                 const json_t *dps_arr)
 {
   json_t *dps;
   unsigned int n_index;
@@ -1239,15 +1228,15 @@ parse_dps_array (struct ANASTASIS_Recovery *r,
   json_array_foreach (dps_arr, n_index, dps)
   {
     struct DecryptionPolicy *dp = &r->dps[n_index];
-    json_t *challenges;
+    const json_t *challenges;
     struct GNUNET_JSON_Specification spec[] = {
       GNUNET_JSON_spec_varsize ("encrypted_master_key",
                                 &dp->emk,
                                 &dp->emk_size),
       GNUNET_JSON_spec_fixed_auto ("master_salt",
                                    &dp->master_salt),
-      GNUNET_JSON_spec_json ("challenges",
-                             &challenges),
+      GNUNET_JSON_spec_array_const ("challenges",
+                                    &challenges),
       GNUNET_JSON_spec_end ()
     };
     const char *err_json_name;
@@ -1270,12 +1259,6 @@ parse_dps_array (struct ANASTASIS_Recovery *r,
     }
     GNUNET_assert (NULL != dp->emk);
     GNUNET_assert (dp->emk_size > 0);
-    if (! json_is_array (challenges))
-    {
-      GNUNET_break_op (0);
-      GNUNET_JSON_parse_free (spec);
-      return GNUNET_SYSERR;
-    }
     dp->pub_details.challenges_length = json_array_size (challenges);
     dp->pub_details.challenges = GNUNET_new_array (
       dp->pub_details.challenges_length,
@@ -1320,8 +1303,7 @@ parse_dps_array (struct ANASTASIS_Recovery *r,
         }
       }
     }
-    /* We don't free the spec, since we're still using dp->ems. */
-    json_decref (challenges);
+    /* Do NOT free the spec: we are still using dp->ems. */
   }
   return GNUNET_OK;
 }
@@ -1363,9 +1345,9 @@ ANASTASIS_recovery_deserialize (struct GNUNET_CURL_Context *ctx,
     const char *err_json_name;
     unsigned int err_line;
     uint32_t version;
-    json_t *dps_arr;
-    json_t *cs_arr;
-    json_t *id_data;
+    const json_t *dps_arr;
+    const json_t *cs_arr;
+    const json_t *id_data;
     const char *provider_url;
     const char *secret_name;
     void *ecs;
@@ -1381,12 +1363,12 @@ ANASTASIS_recovery_deserialize (struct GNUNET_CURL_Context *ctx,
         NULL),
       GNUNET_JSON_spec_uint32 ("version",
                                &version),
-      GNUNET_JSON_spec_json ("decryption_policies",
-                             &dps_arr),
-      GNUNET_JSON_spec_json ("challenges",
-                             &cs_arr),
-      GNUNET_JSON_spec_json ("id_data",
-                             &id_data),
+      GNUNET_JSON_spec_array_const ("decryption_policies",
+                                    &dps_arr),
+      GNUNET_JSON_spec_array_const ("challenges",
+                                    &cs_arr),
+      GNUNET_JSON_spec_object_const ("id_data",
+                                     &id_data),
       GNUNET_JSON_spec_varsize ("encrypted_core_secret",
                                 &ecs,
                                 &ecs_size),
@@ -1420,7 +1402,7 @@ ANASTASIS_recovery_deserialize (struct GNUNET_CURL_Context *ctx,
       GNUNET_JSON_parse_free (spec);
       return NULL;
     }
-    r->id_data = json_incref (id_data);
+    r->id_data = json_incref ((json_t *) id_data);
     r->provider_url = GNUNET_strdup (provider_url);
     if (NULL != secret_name)
       r->secret_name = GNUNET_strdup (secret_name);
