@@ -106,6 +106,9 @@ handle_policy_lookup_finished (void *cls,
                                size_t data_size)
 {
   struct ANASTASIS_PolicyLookupOperation *plo = cls;
+  struct ANASTASIS_DownloadDetails dd = {
+    .http_status = response_code
+  };
 
   plo->job = NULL;
   switch (response_code)
@@ -117,7 +120,6 @@ handle_policy_lookup_finished (void *cls,
     break;
   case MHD_HTTP_OK:
     {
-      struct ANASTASIS_DownloadDetails dd;
       struct ANASTASIS_UploadSignaturePS usp = {
         .purpose.purpose = htonl (TALER_SIGNATURE_ANASTASIS_POLICY_UPLOAD),
         .purpose.size = htonl (sizeof (usp)),
@@ -133,18 +135,17 @@ handle_policy_lookup_finished (void *cls,
                                       &plo->account_pub.pub))
       {
         GNUNET_break_op (0);
-        response_code = 0;
+        dd.http_status = 0;
+        dd.ec = -1; // FIXME: needs new code in Gana!
         break;
       }
       /* Success, call callback with all details! */
-      memset (&dd, 0, sizeof (dd));
-      dd.sig = plo->account_sig;
-      dd.curr_policy_hash = usp.new_recovery_data_hash;
-      dd.policy = data;
-      dd.policy_size = data_size;
-      dd.version = plo->version;
+      dd.details.ok.sig = plo->account_sig;
+      dd.details.ok.curr_policy_hash = usp.new_recovery_data_hash;
+      dd.details.ok.policy = data;
+      dd.details.ok.policy_size = data_size;
+      dd.details.ok.version = plo->version;
       plo->cb (plo->cb_cls,
-               response_code,
                &dd);
       plo->cb = NULL;
       ANASTASIS_policy_lookup_cancel (plo);
@@ -167,12 +168,10 @@ handle_policy_lookup_finished (void *cls,
                 "Unexpected response code %u\n",
                 (unsigned int) response_code);
     GNUNET_break (0);
-    response_code = 0;
     break;
   }
   plo->cb (plo->cb_cls,
-           response_code,
-           NULL);
+           &dd);
   plo->cb = NULL;
   ANASTASIS_policy_lookup_cancel (plo);
 }
