@@ -72,32 +72,26 @@ struct PolicyLookupState
  * Function called with the results of a #ANASTASIS_policy_lookup().
  *
  * @param cls closure
- * @param http_status HTTP status of the request
  * @param dd details about the lookup operation
  */
 static void
 policy_lookup_cb (void *cls,
-                  unsigned int http_status,
                   const struct ANASTASIS_DownloadDetails *dd)
 {
   struct PolicyLookupState *pls = cls;
 
   pls->plo = NULL;
-  if (http_status != pls->http_status)
+  if (dd->http_status != pls->http_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u to command %s in %s:%u\n",
-                http_status,
-                pls->is->commands[pls->is->ip].label,
-                __FILE__,
-                __LINE__);
-    TALER_TESTING_interpreter_fail (pls->is);
+    TALER_TESTING_unexpected_status (pls->is,
+                                     dd->http_status,
+                                     pls->http_status);
     return;
   }
   if (NULL != pls->upload_reference)
   {
-    if ( (MHD_HTTP_OK == http_status) &&
-         (0 != GNUNET_memcmp (&dd->curr_policy_hash,
+    if ( (MHD_HTTP_OK == dd->http_status) &&
+         (0 != GNUNET_memcmp (&dd->details.ok.curr_policy_hash,
                               pls->upload_hash)) )
     {
       GNUNET_break (0);
@@ -156,11 +150,12 @@ policy_lookup_run (void *cls,
     }
     pls->anastasis_pub = *anastasis_pub;
   }
-  pls->plo = ANASTASIS_policy_lookup (is->ctx,
-                                      pls->anastasis_url,
-                                      &pls->anastasis_pub,
-                                      &policy_lookup_cb,
-                                      pls);
+  pls->plo = ANASTASIS_policy_lookup (
+    TALER_TESTING_interpreter_get_context (is),
+    pls->anastasis_url,
+    &pls->anastasis_pub,
+    &policy_lookup_cb,
+    pls);
   if (NULL == pls->plo)
   {
     GNUNET_break (0);
