@@ -125,16 +125,11 @@ truth_store_cb (void *cls,
   struct TruthStoreState *tss = cls;
 
   tss->tso = NULL;
-  if ( (NULL == ud) ||
-       (ud->http_status != tss->http_status) )
+  if (ud->http_status != tss->http_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u to command %s in %s:%u\n",
-                (NULL != ud) ? ud->http_status : 0,
-                tss->is->commands[tss->is->ip].label,
-                __FILE__,
-                __LINE__);
-    TALER_TESTING_interpreter_fail (tss->is);
+    TALER_TESTING_unexpected_status (tss->is,
+                                     ud->http_status,
+                                     tss->http_status);
     return;
   }
   switch (ud->us)
@@ -273,7 +268,7 @@ truth_store_run (void *cls,
       GNUNET_free (t);
     }
     tss->tso = ANASTASIS_truth_store (
-      is->ctx,
+      TALER_TESTING_interpreter_get_context (is),
       tss->anastasis_url,
       &tss->uuid,
       tss->method,
@@ -345,10 +340,8 @@ truth_store_traits (void *cls,
     ANASTASIS_TESTING_make_trait_truth_key (&tss->key),
     ANASTASIS_TESTING_make_trait_eks (&tss->encrypted_keyshare),
     ANASTASIS_TESTING_make_trait_payment_secret (&tss->payment_secret_response),
-    TALER_TESTING_make_trait_payto_uri (
-      (const char **) &tss->pay_uri),
-    ANASTASIS_TESTING_make_trait_filename (
-      (const char **) &tss->filename),
+    TALER_TESTING_make_trait_payto_uri (tss->pay_uri),
+    ANASTASIS_TESTING_make_trait_filename (tss->filename),
     TALER_TESTING_trait_end ()
   };
 
