@@ -135,14 +135,7 @@ secret_share_result_cb (void *cls,
   sss->sso = NULL;
   if (sr->ss != sss->want_status)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u to command %s in %s:%u\n",
-                sr->ss,
-                sss->is->commands[sss->is->ip].label,
-                __FILE__,
-                __LINE__);
     TALER_TESTING_interpreter_fail (sss->is);
-    return;
   }
   switch (sr->ss)
   {
@@ -225,7 +218,7 @@ secret_share_run (void *cls,
   if (NULL != sss->prev_secret_share)
   {
     const struct TALER_TESTING_Command *ref;
-    const char **order_id;
+    const char *order_id;
 
     ref = TALER_TESTING_interpreter_lookup_command (is,
                                                     sss->prev_secret_share);
@@ -243,8 +236,7 @@ secret_share_run (void *cls,
       TALER_TESTING_interpreter_fail (sss->is);
       return;
     }
-    sss->payment_order_id = (char *) *order_id;
-
+    sss->payment_order_id = GNUNET_strdup (order_id);
     if (NULL == sss->payment_order_id)
     {
       GNUNET_break (0);
@@ -296,19 +288,20 @@ secret_share_run (void *cls,
     pds.provider_salt = *provider_salt;
   }
 
-  sss->sso = ANASTASIS_secret_share (is->ctx,
-                                     sss->id_data,
-                                     &pds,
-                                     1,
-                                     policies,
-                                     sss->cmd_label_array_length,
-                                     false,
-                                     GNUNET_TIME_UNIT_ZERO,
-                                     &secret_share_result_cb,
-                                     sss,
-                                     "test-case",
-                                     sss->core_secret,
-                                     sss->core_secret_size);
+  sss->sso = ANASTASIS_secret_share (
+    TALER_TESTING_interpreter_get_context (is),
+    sss->id_data,
+    &pds,
+    1,
+    policies,
+    sss->cmd_label_array_length,
+    false,
+    GNUNET_TIME_UNIT_ZERO,
+    &secret_share_result_cb,
+    sss,
+    "test-case",
+    sss->core_secret,
+    sss->core_secret_size);
   if (NULL == sss->sso)
   {
     GNUNET_break (0);
@@ -366,9 +359,8 @@ secret_share_traits (void *cls,
   struct SecretShareState *sss = cls;
   struct TALER_TESTING_Trait traits[] = {
     TALER_TESTING_make_trait_claim_token (&sss->token),
-    ANASTASIS_TESTING_make_trait_core_secret (&sss->core_secret),
-    TALER_TESTING_make_trait_order_id (
-      (const char **) &sss->payment_order_id),
+    ANASTASIS_TESTING_make_trait_core_secret (sss->core_secret),
+    TALER_TESTING_make_trait_order_id (sss->payment_order_id),
     TALER_TESTING_trait_end ()
   };
 
