@@ -208,19 +208,20 @@ truth_upload_run (void *cls,
   ANASTASIS_CRYPTO_user_identifier_derive (tus->id_data,
                                            provider_salt,
                                            &user_id);
-  tus->tuo = ANASTASIS_truth_upload (is->ctx,
-                                     &user_id,
-                                     tus->anastasis_url,
-                                     tus->method,
-                                     tus->instructions,
-                                     tus->mime_type,
-                                     provider_salt,
-                                     tus->truth_data,
-                                     tus->truth_data_size,
-                                     false, /* force payment */
-                                     GNUNET_TIME_UNIT_ZERO,
-                                     &truth_upload_cb,
-                                     tus);
+  tus->tuo = ANASTASIS_truth_upload (
+    TALER_TESTING_interpreter_get_context (is),
+    &user_id,
+    tus->anastasis_url,
+    tus->method,
+    tus->instructions,
+    tus->mime_type,
+    provider_salt,
+    tus->truth_data,
+    tus->truth_data_size,
+    false,                                  /* force payment */
+    GNUNET_TIME_UNIT_ZERO,
+    &truth_upload_cb,
+    tus);
   if (NULL == tus->tuo)
   {
     GNUNET_break (0);
